@@ -4,18 +4,21 @@
 from __future__ import annotations
 
 import logging
+from typing import Optional
+from collections.abc import Sequence
 
+import dns.dnssec
+import dns.message
 import dns.query
 import dns.resolver
-import dns.message
-import dns.dnssec
+import dns.rdatatype
+import dns.name
+from dns.nameserver import Nameserver
 from dns.rdatatype import RdataType
-
 from expiringdict import ExpiringDict
 
-from checkdmarc.utils import normalize_domain, get_base_domain
-from checkdmarc._constants import DNSSEC_CACHE_MAX_LEN, DNSSEC_CACHE_MAX_AGE_SECONDS
-
+from checkdmarc._constants import DNSSEC_CACHE_MAX_AGE_SECONDS, DNSSEC_CACHE_MAX_LEN
+from checkdmarc.utils import get_base_domain, normalize_domain
 
 """Copyright 2019-2023 Sean Whalen
 
@@ -45,10 +48,10 @@ TLSA_CACHE = ExpiringDict(
 def get_dnskey(
     domain: str,
     *,
-    nameservers: list[str] = None,
+    nameservers: Optional[Sequence[str | Nameserver]] = None,
     timeout: float = 2.0,
-    cache: ExpiringDict = None,
-):
+    cache: Optional[ExpiringDict] = None,
+) -> Optional[dict]:
     """
     Get a DNSKEY RRSet on the given domain
 
@@ -59,7 +62,7 @@ def get_dnskey(
         cache (ExpiringDict): A cache
 
     Returns:
-        A DNSKEY dictionary
+        A DNSKEY dictionary if a DNSKEY is found
     """
     if nameservers is None:
         nameservers = dns.resolver.Resolver().nameservers
@@ -69,13 +72,15 @@ def get_dnskey(
     domain = normalize_domain(domain)
 
     if domain in cache:
-        return cache[domain]
+        cached_result = cache[domain]
+        if isinstance(cached_result, dict):
+            return cached_result
 
     logging.debug(f"Checking for DNSKEY records at {domain}")
     request = dns.message.make_query(domain, dns.rdatatype.DNSKEY, want_dnssec=True)
     for nameserver in nameservers:
         try:
-            response = dns.query.tcp(request, nameserver, timeout=timeout)
+            response = dns.query.tcp(request, str(nameserver), timeout=timeout)
             if response is not None:
                 answer = response.answer
                 if len(answer) == 0:
@@ -104,9 +109,9 @@ def get_dnskey(
 def test_dnssec(
     domain: str,
     *,
-    nameservers: list[str] = None,
+    nameservers: Optional[Sequence[str | Nameserver]] = None,
     timeout: float = 2.0,
-    cache: ExpiringDict = None,
+    cache: Optional[ExpiringDict] = None,
 ) -> bool:
     """
     Check for DNSSEC on the given domain
@@ -126,7 +131,9 @@ def test_dnssec(
         cache = DNSSEC_CACHE
 
     if domain in cache:
-        return cache[domain]
+        cached_result = cache[domain]
+        if isinstance(cached_result, bool):
+            return cached_result
 
     key = get_dnskey(domain, nameservers=nameservers, timeout=timeout)
     if key is None:
@@ -142,7 +149,7 @@ def test_dnssec(
         request = dns.message.make_query(domain, rdatatype, want_dnssec=True)
         for nameserver in nameservers:
             try:
-                response = dns.query.tcp(request, nameserver, timeout=timeout)
+                response = dns.query.tcp(request, str(nameserver), timeout=timeout)
                 if response is not None:
                     answer = response.answer
                     if len(answer) != 2:
@@ -168,11 +175,11 @@ def test_dnssec(
 def get_tlsa_records(
     hostname: str,
     *,
-    nameservers: list[str] = None,
+    nameservers: Sequence[str | Nameserver],
     timeout: float = 2.0,
     port: int = 25,
     protocol: str = "tcp",
-    cache: ExpiringDict = None,
+    cache: Optional[ExpiringDict] = None,
 ) -> list[str]:
     """
     Checks for TLSA records on the given hostname
@@ -195,16 +202,21 @@ def get_tlsa_records(
         cache = TLSA_CACHE
 
     query_hostname = f"_{port}._{protocol}.{hostname}"
-    if query_hostname in TLSA_CACHE:
-        return TLSA_CACHE[query_hostname]
-    tlsa_records = []
+    if isinstance(cache, ExpiringDict):
+        if query_hostname in TLSA_CACHE:
+            cached_results = TLSA_CACHE[query_hostname]
+            if isinstance(cached_results, list):
+                return cached_results
+    tlsa_records: list[str] = []
     logging.debug(f"Checking for TLSA records at {query_hostname}")
     request = dns.message.make_query(
         query_hostname, dns.rdatatype.TLSA, want_dnssec=True
     )
+    if len(nameservers) == 0:
+        raise ValueError("At lease one nameserver is required")
     for nameserver in nameservers:
         try:
-            response = dns.query.tcp(request, nameserver, timeout=timeout)
+            response = dns.query.tcp(request, str(nameserver), timeout=timeout)
             if response is not None:
                 answer = response.answer
                 if len(answer) != 2:
@@ -225,10 +237,12 @@ def get_tlsa_records(
                         rrsig = rset
                     else:
                         rrset = rset
-                dns.dnssec.validate(rrset, rrsig, dnskey)
-                tlsa_records = list(map(lambda x: str(x), list(rrset.items.keys())))
-                cache[query_hostname] = tlsa_records
+                if rrset is not None:
+                    dns.dnssec.validate(rrset, rrsig, dnskey)
+                    tlsa_records = list(map(lambda x: str(x), list(rrset.items.keys())))
+                    cache[query_hostname] = tlsa_records
                 return tlsa_records
         except Exception as e:
             logging.debug(f"TLSA query error: {e}")
             return tlsa_records
+    return tlsa_records
